﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Runtime.Serialization.Formatters.Binary;
using System.Windows;
using ThinkGeo.Core;

namespace Demo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow
    {
        private GridCell[,] _gridCellMatrix;
        private readonly string[] _colorStrings = {
            "#333333",
            "#000080",
            "#0000ff",
            "#00ffff",
            "#339a65",
            "#ffff00",
            "#ff9a00",
        };

        readonly List<double> _isoLineLevels = new List<double>
        {
            16.67 ,
            32.34 ,
            48.01 ,
            63.68 ,
            15.0  ,
            79.35 ,
            95    ,
        };
        

        public MainWindow()
        {
            InitializeComponent();
        }

        private void MapView_OnLoaded(object sender, RoutedEventArgs e)
        {
            mapView.MapUnit = GeographyUnit.Meter;
            GetGridIsoLineLayer(IsoLineType.ClosedLinesAsPolygons);
            mapView.Refresh();
        }

        

        private void CreateGridCellMatrix()
        {
            var gridDefinition = BinaryDeserialize<GridDefinition>("data/Grid.grid");
            mapView.CurrentExtent = gridDefinition.GridExtent;
            ///////////////////////////////////////todo uncomment this to resolve the issue...//////////////////////////////////////
            //foreach (var dataPointsKey in gridDefinition.DataPoints.Keys.ToList())
            //{
            //    gridDefinition.DataPoints[dataPointsKey] = Math.Round(gridDefinition.DataPoints[dataPointsKey], 2);
            //}
            ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
            _gridCellMatrix = GridFeatureSource.GenerateGridMatrix(gridDefinition, new InverseDistanceWeightedGridInterpolationModel(5, gridDefinition.CellSize * 4));
        }

        

        public static T BinaryDeserialize<T>(string filename)
        {
            var fileStream = new FileStream(filename, FileMode.Open);
            try
            {
                return (T)new BinaryFormatter().Deserialize(fileStream);
            }
            finally
            {
                fileStream.Close();
            }
        }

        private void GetGridIsoLineLayer(IsoLineType type)
        {
            CreateGridCellMatrix();

            var isoLineLayer = new SqlTypesIsoLineLayer(_gridCellMatrix, _isoLineLevels)
            {
                IsoLineType = type
            };
            var features = isoLineLayer.GetIsoLineFeatures();
            var overlay = new LayerOverlay();

            mapView.Overlays.Add($"isoLineOverlay", overlay);

            for (var i = 0; i < _isoLineLevels.Count; i++)
            {
                var ff = new InMemoryFeatureLayer();
                overlay.Layers.Add(ff);
                var set = new ZoomLevelSet();
                ff.ZoomLevelSet = set;
                set.ZoomLevel01.DefaultAreaStyle = new AreaStyle(new GeoPen(GeoColors.LightBlue, 1),
                    new GeoSolidBrush(GeoColor.FromArgb(80, GeoColor.FromHtml(_colorStrings[i]))));
                set.ZoomLevel01.DefaultLineStyle =
                    new LineStyle(new GeoPen(GeoColor.FromHtml(_colorStrings[i]), 3));

                set.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
                foreach (var feature in features.Where(f =>
                {
                    var value = double.Parse(f.ColumnValues["ContourValue"]);
                    return value >= (i == 0 ? 0 : _isoLineLevels[i - 1]) && value < _isoLineLevels[i];
                }))
                {
                    ff.InternalFeatures.Add(feature);
                }
            }
        }
    }
}
